@@ -1,4 +1,7 @@
 """The Nim Game calculation functions
+
+This module defines the functions that are needed to calculate moves and
+decisions in a :class:`Mixin` for the :class:`core.Nim` class.
 """
 
 
@@ -7,74 +10,14 @@ import random
 from source.typedefs import Move, ErrorRate
 
 
-class EndGameCheck():
-    """Class for handling the end-game if it is misere type
-    
-    The instance can be checked in a bool context, indicating whether the status
-    is the beginning of the end-game.
-    
-    Methods:
-    - check_heap() gets a heap index and number of coins in the heap. It then
-        sets the internal status attributes (see below). This method is called
-        in a loop for all heaps and then provides the status of the game
-        indicating whether it is the beginning of the end-game.
-    - get_move() returns the required move at the beginning of end-game
-    
-    Attributes:
-    - one_manycoins: Indicator whether all active heaps are single-coin, but
-        one. This is when the end-game starts.
-    - even_singlecoins: Indicator whether the number of single-coin heaps is
-        even or odd. This is needed to figure out the move at the beginning of
-        the end-game.
-    """
-    one_manycoins = None
-    even_singlecoins = True
-    
-    def __bool__(self):
-        return bool(self.one_manycoins)
-    
-    def check_heap(self,
-                idx: int, 
-                heapcount: int, 
-    ) -> None:
-        if heapcount>1:
-            if self.one_manycoins is None:
-                self.one_manycoins = True
-                self.heapcount = heapcount
-                self.heapidx = idx
-            else:
-                self.one_manycoins = False
-        
-        elif heapcount==1:
-            self.even_singlecoins = not self.even_singlecoins
-    
-    def get_move(self) -> Move:
-        #even number of heaps with 1 coin?
-        if self.even_singlecoins:
-            #remove all but 1 coin from the heap which has more coins
-            removecount = self.heapcount-1
-        else:
-            #remove all coins from the heap which has more coins
-            removecount = self.heapcount
-        #remove coins so that odd number of 1-coin heaps remain
-        return Move(self.heapidx, removecount)
-
-
 class Mixin:
     """Class to implement the Nim game calculations
-
-    Methods:
-    - make_good_choice(): figure out whether to make a good decision
-    - get_nimsum(): 'nimsum' is a special sum of the heaps
-    - all_one_heaps(): it indicates whether all heaps have either 1 or zero
-        coins
-    - is_winning_for_next(): it indicates whether the current status is good for
-        the player having the next turn
-    - get_random_move(): just a utility to pick a heap and coins randomly
-    - figure_out_best_move(): intelligently figure out the next best move
+    
+    This Mixin is used as an ancestor class for :class:`core.Nim`, so
+    that we can define methods here, in a separate module.
     """
     def make_good_choice(self) -> bool:
-        """Figure out whether the good choise is to be made
+        """Figures out whether to make a good decision or make an error
 
         The required error rate indicates how bad decisions the Computer is to
         make. The higher the error rate percentage the higher the possibility
@@ -104,8 +47,8 @@ class Mixin:
     def get_nimsum(self) -> int:
         """Calcualte the nimsum of the set of heaps
 
-        Use the functools.reduce() on the heaps list, with a 0 initializing
-        value and use XOR for the coin counts of the heaps.
+        It uses the functools.reduce() on the heaps list, with a 0 initializing
+        value and uses XOR for the coin counts of the heaps.
 
         Returns:
             The nimsum of the heap-set
@@ -118,10 +61,11 @@ class Mixin:
 
 
     def all_one_heaps(self) -> bool:
-        """Check whether all non-zero heaps have 1 single coin
+        """Checks whether all non-zero heaps have 1 single coin
 
-        Use the functools.reduce() on the heaps list, with a True initializing
-        value and keep it true as long as the number of coins is at most one.
+        It uses the functools.reduce() on the heaps list, with a True
+        initializing value and keep it true as long as the number of coins is at
+        most one.
 
         Returns:
             Whether all relevant heaps have 1 single coin
@@ -147,7 +91,8 @@ class Mixin:
         nimsum. The boolean reversal is done with an XOR.
         
         Returns:
-            Whether the current status good for the player having the next turn
+            Whether the current status is good for the player having the next
+            turn
         """
         if self.misere:
             return bool(self.get_nimsum()) ^ self.all_one_heaps()
@@ -158,12 +103,12 @@ class Mixin:
     def get_random_move(self) -> Move:
         """Figure out a random but valid move
         
-        This method is used when the algorithm does not care what to move, just
-        figure out a random move.
+        This method is used when the algorithm does not care what to move. Just
+        figure out a random but valid move.
         
         Returns:
             From which heap and how many coins are to be removed in the coming
-                move
+            move
         """
         #pick a random index from the active (i.e. non-empty) heaps
         heapnumber = random.choice(
@@ -198,32 +143,59 @@ class Mixin:
         hope that the opponent makes a mistake.
 
         If nimsum is not zero, it can be made zero by removing a cretain number
-        of coins from the right heap. Check each heap first calculating a
-        "target" number of coins, which would null the nimsum. E.g. if the
-        nimsum is 7, i.e. we need to remove a binary 4, 2 and 1, and the heap
-        contains 9 coins, the target would be 9 XOR 7 = 14. However, we cannot
-        remove 14 coins from 9, i.e. this is not a suitable heap to remove
-        coins from. So, go on until finding a suitable heap.
+        of coins from the right heap. Check each heap and calculate the "target"
+        number of coins, to null the nimsum. E.g. if the nimsum is 7 (binary
+        111), we need to remove all 3 binary digits, i.e. the 4, 2 and 1,
+        without changing any other binary digits, e.g. the 8. If a given heap
+        contains e.g. 9 coins (binary 1001), the "target" would be the
+        :math:`9 xor 7`::
+        
+            1001
+            0111
+            ----
+            1110 => decimal 14
+        
+        As we cannot remove 14 coins from a heap where there are 9 coins, this
+        is not a suitable heap to remove coins from. So, go on until finding a
+        suitable heap. Theoretically, there must be at least 1 suitable heap.
 
         Returns:
             From which heap and how many coins are to be removed in the coming
-                move
+            move
         """
-        #first check whether all active heaps have 1 coin only
+        #active heaps have 1 coin only?
         if self.all_one_heaps():
             #empty one of the 1-coin heaps, does not matter which one
             return self.get_random_move()
 
         #need to handle the "only 1 heap with more than 1 coin" situation?
         if self.misere:
-            #create an instance to analyse the heap for end-game
-            endgamecheck = EndGameCheck()
+            #number of bigger heaps
+            bigheapcount = 0
+            #by default the number of 1-coin heaps is even (NB. zero is even)
+            even_singlecoins = True
+            #loop on all heaps
             for idx, heapcount in enumerate(self.heaps):
-                endgamecheck.check_heap(idx, heapcount)
+                #big heap?
+                if heapcount>1:
+                    bigheapcount += 1
+                    bigheapcontent = heapcount
+                    bigheapidx = idx
+                
+                elif heapcount==1:
+                    even_singlecoins = not even_singlecoins
             
             #entering end-game, i.e. only 1 heap with more than 1 coin?
-            if endgamecheck:
-                return endgamecheck.get_move()
+            if bigheapcount==1:
+                #even number of heaps with 1 coin?
+                if even_singlecoins:
+                    #remove all but 1 coin from the heap which has more coins
+                    removecount = bigheapcontent-1
+                else:
+                    #remove all coins from the heap which has more coins
+                    removecount = bigheapcontent
+                #remove coins so that odd number of 1-coin heaps remain
+                return Move(bigheapidx, removecount)
 
         #calcualte the nimsum of the set of heaps
         nimsum = self.get_nimsum()
