@@ -1,4 +1,4 @@
-"""Play interactive Nim Game in the CLI
+"""Play an interactive Nim Game in the CLI
 """
 
 
@@ -12,7 +12,7 @@ def play_CLI() -> None:
 
     Create random heaps first. Then, get the user to select who to start.
     Player and Computer take turns until the game ends. Computer will make good
-    or bad decisions based on the error rate set in the inctance attribute.
+    or bad decisions based on the error rate set in the instance attribute.
     """
     #create instance and random heaps, then display them
     nim = Nim()
